{
--------------------------------------------------------------------------------

Component Name: TLEDArray
Author: Mats Asplund
Creation: June 28, 1999
Version: 1.0
Description:  TLEDArray is a component for displaying a row of LEDs.
Credit:
EMail:        mats.asplund@telia.com
Site:         http://w1.545.telia.com/~u54503556/delphi/mdp.htm
Legal issues: Copyright (C) 1999 by Mats Asplund
              This software is provided 'as-is', without any express or
              implied warranty.  In no event will the author be held liable
              for any  damages arising from the use of this software.

              Permission is granted to anyone to use this software for any
              purpose, including commercial applications, and to alter it
              and redistribute it freely, subject to the following
              restrictions:

              1. The origin of this software must not be misrepresented,
                 you must not claim that you wrote the original software.
                 If you use this software in a product, an acknowledgment
                 in the product documentation would be appreciated but is
                 not required.

              2. Altered source versions must be plainly marked as such, and
                 must not be misrepresented as being the original software.

              3. This notice may not be removed or altered from any source
                 distribution.

              4. If you decide to use this software in any of your applications.
                 Send me an EMail address and tell me about it.

Quick Reference:
              TDLBox is a descendant from TGraphicControl.

              Published properties:
              StyleChanged: Repaints the component.
              NoOfLEDs:     How many LED:s to be displayed. Maximized to 32.
              Space:        Space between LED:s.
              OnOff:        Sets the LED:s on/off. See the Led-positions as
                            binary bit positions. For example a value of 255
                            will turn all LED:s on in an array with 8 LED:s.
              Brush:        Sets the brush properties.
              Pen:          Sets the pen properties.
              Shape:        Sets the shape.
              StyleChanged: Repaints the component.

              All others properties and events are inherited.
--------------------------------------------------------------------------------
}

unit ledarray;

interface

uses WinTypes, WinProcs, Messages, SysUtils, Classes, Controls,
     Forms, Graphics;

type
  TShapeType = (stRectangle, stSquare, stRoundRect, stRoundSquare,
    stEllipse, stCircle);

  TLEDArray = class(TGraphicControl)
    private
      FAntal, FSpace, FX, FY: integer;
      FOnOff: cardinal;
      FPen: TPen;
      FBrush: TBrush;
      FShape: TShapeType;
      FOm: string;
      procedure SetAntal(Value: integer);
      procedure SetSpace(Value: integer);
      procedure SetOnOff(Value: cardinal);
      procedure SetBrush(Value: TBrush);
      procedure SetPen(Value: TPen);
      procedure SetShape(Value: TShapeType);
      procedure WMSize(var Message: TWMSize); message WM_SIZE;
      function GetOm: string;
      procedure SetOm(Value: string);

    protected
      procedure Paint; override;
      procedure Paint2(X, Y: integer);
      procedure Click; override;
      procedure Loaded; override;
    public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
    published
      procedure StyleChanged(Sender: TObject);
      property NoOfLEDs: integer read FAntal write SetAntal;
      property Space: integer read FSpace write SetSpace;
      property OnOff: cardinal read FOnOff write SetOnOff;
      property Align;
      property Anchors;
      property Brush: TBrush read FBrush write SetBrush;
      property DragCursor;
      property DragKind;
      property DragMode;
      property Enabled;
      property Constraints;
      property ParentShowHint;
      property Pen: TPen read FPen write SetPen;
      property Shape: TShapeType read FShape write SetShape default stRectangle;
      property ShowHint;
      property Visible;
      property Copyright: string read GetOm write SetOm;
      property OnDragDrop;
      property OnDragOver;
      property OnEndDock;
      property OnEndDrag;
      property OnMouseDown;
      property OnMouseMove;
      property OnMouseUp;
      property OnStartDock;
      property OnStartDrag;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MAs Prod.', [TLEDArray]);
end;

constructor TLEDArray.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  Width := 156;
  Height := 16;
  FSpace:=20;
  FAntal:=8;
  FOnOff:=255;
  FX:=0;
  FY:=0;
  FPen := TPen.Create;
  FPen.OnChange := StyleChanged;
  FBrush := TBrush.Create;
  FBrush.OnChange := StyleChanged;
  Brush.Color:=clLime;
  Shape:=stCircle;
  FOm:='1999 Mats Asplund/MAs Prod.';
end;

destructor TLEDArray.Destroy;
begin
  FPen.Free;
  FBrush.Free;
  inherited Destroy;
end;

procedure TLEDArray.Paint;
var n, Till: integer;
    TF: cardinal;
begin
  TF:=FOnOff;
  FX:=0;
  for n:=1 to FAntal do
  begin
    Till:=TF and 1;
    if Till = 1 then Paint2(FX,0);
    FX:=FX+FSpace;
    TF:=TF shr 1;
  end;
end;

procedure TLEDarray.Paint2(X, Y: integer);
var
   W, H, S: Integer;
begin
  with Canvas do
  begin
      Pen := FPen;
      Brush := FBrush;
      W := Width - Pen.Width + 1;
      H := Height - Pen.Width + 1;
      if Pen.Width = 0 then
      begin
        Dec(W);
        Dec(H);
      end;
      if W < H then S := W else S := H;
      if FShape in [stSquare, stRoundSquare, stCircle] then
      begin
        W := S;
        H := S;
      end;
      case FShape of
        stRectangle, stSquare:
          Rectangle(X, Y, X + W, Y + H);
        stRoundRect, stRoundSquare:
          RoundRect(X, Y, X + W, Y + H, S div 4, S div 4);
        stCircle, stEllipse:
          Ellipse(X, Y, X + W, Y + H);
      end;
  end;
  if (W >= FSpace) and (FShape = stCircle) then SetSpace(W+1);
end;

procedure TLEDArray.StyleChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TLEDArray.SetBrush(Value: TBrush);
begin
  FBrush.Assign(Value);
end;

procedure TLEDArray.SetPen(Value: TPen);
begin
  FPen.Assign(Value);
end;

procedure TLEDarray.SetShape(Value: TShapeType);
begin
  if FShape <> Value then
  begin
    FShape := Value;
    Width := 156;
    Height := 16;
    FSpace:=20;
    Invalidate;
  end;
end;

procedure TLEDArray.Click;
begin
  inherited Click;
end;


procedure TLEDArray.Loaded;
begin
  inherited Loaded;
end;

procedure TLEDArray.WMSize(var Message: TWMSize);
var
     W, H: Integer;
begin
  inherited;
  W := Width;
  H := Height;
  if (W <> Width) or (H <> Height) then
  inherited SetBounds(Left, Top, W, H);
  Message.Result := 0;
end;


procedure TLEDArray.SetAntal(Value: integer);
begin
  FAntal:=Value;
  if FAntal > 32 then FAntal:=32;
  Width:=FAntal*FSpace;
  Paint;
end;

procedure TLEDArray.SetSpace(Value: integer);
begin
  FSpace:=Value;
  Width:=FAntal*FSpace;
  Paint;
end;

procedure TLEDArray.SetOnOff(Value: cardinal);
begin
  FOnOff:=Value;
  Width:=0;
  Paint;
  SetAntal(FAntal);
end;

function TLEDArray.GetOm: string;
begin
  Result:=FOm;
end;

procedure TLEDArray.SetOm(Value: string);
begin
  FOm:=FOm;
end;

end.
